#ifndef ISTL_SOCKET_INCLUDE
#define ISTL_SOCKET_INCLUDE


#if defined(_WIN32) && !defined(__CYGWIN__)
#	include <winsock2.h>
#	define _USE_WINSOCK
#	pragma comment(lib,"ws2_32.lib")
#else
#	include <sys/socket.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include <netdb.h>
#	include <unistd.h>
#	define _USE_BSDSOCK
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



namespace ist {


#ifdef _USE_WINSOCK
	typedef SOCKET socket_t;
	static const int socket_error=SOCKET_ERROR;
#else
	typedef int socket_t;
	static const int socket_error=-1;
#endif

	class Socket
	{
	public:
		explicit Socket(socket_t s=-1);
		Socket(const char *host, unsigned short port);
		~Socket();

		bool open(const char *host, unsigned short port);
		bool open(int type, const char *host, unsigned short port);
		void close();

		bool write(char c);
		bool write(const char *str, int length);
		char read();			// G[yeof̖߂l0
		int  read(char *c, int length);	// G[-1Aeof0

		socket_t get() const { return _sock; }
		bool is_opened() const;
		bool operator !() const;

	private:
		socket_t _sock;
	};


	class ServerSocket
	{
	public:
		ServerSocket();
		ServerSocket(int port, int backlog=5);
		ServerSocket(int type, int port, int backlog=5);
		~ServerSocket();

		bool bind(int port, int backlog=5);
		socket_t accept();
		void close();

		socket_t get() const { return _sock; }
		bool operator !() const;

	private:
		socket_t _sock;
		bool _is_listened;
	};

}

#endif
