#ifndef IST_SPLINE
#define IST_SPLINE

#include <vector>
#include <utility>
#include "vector.h"


namespace ist {

class Spline2D
{
public:
	struct point_t
	{
		point_t() {}
		point_t(const vector2& p) : in(p), pos(p), out(p) {}
		point_t(const vector2& i, const vector2& p, const vector2& o) : in(i), pos(p), out(o) {}

		vector2 in, pos, out;
	};
	typedef std::vector<Spline2D::point_t> point_cont;
	typedef point_cont::iterator iterator;
	typedef std::pair<iterator, iterator> range_t;


	Spline2D() {}
	size_t size() const { return points.size(); }
	iterator begin() { return points.begin(); }
	iterator end()   { return points.end(); }
	void erase(iterator itr) { points.erase(itr); }
	void clear() { points.clear(); }

	range_t get_edge(float x);
	void insert(const vector2& p);
	void insert(const vector2& i, const vector2& p, const vector2& o);
	vector2 linear(float x);
	vector2 bezier(float x);

private:
	point_cont points;
};

} // ist

#endif
